% Numerical example 1 in the paper:
%
%   [1] Ding Lu, Yangfeng Su, and Zhaojun Bai, Stability Analysis 
%       of the two-level orthogonal Arnoldi procedure,
%       SIAM J. Matrix Anal. Appl., 2016. 37(1): 195--214.
%
% Ding Lu, Fudan Univ., 2012/2/8.
%
clc; clear all; close all;

load y6103f.mat; E =D;

N = size(M,1); % problem size
fq = [linspace(0,1000,300),linspace(1000,3000,300)]; % frequency range
s = 2*pi*sqrt(-1) *fq;
s0 = 150*2*pi; % expansion point

h = zeros( size(s) ); % exact transfor function
for i = 1:length(s)
    h(i) = l'*( (s(i)*s(i)*M+s(i)*E+K)\b );
end

% Shifting coefficient matrices
if s0==0
    TM = M; TE = E; TK = K;
else
    TM = M; TE = 2*s0*M + E; TK = s0^2*M + s0*E + K;
end

[L,U,P] = lu(TK); % LU decomp

% Generate Tb, Tl
Tb = [b;zeros(size(b))];
Tl = [l;zeros(size(l))];

hhs=[]; 
hht=[];
for n = [10,20,40]
    v0 = U\(L\(P*b));      
    av = @(p) -U\(L\(P*(TE*p)));
    bv = @(q) -U\(L\(P*(TM*q))); 

    % MOR by TOAR 
    [Vn, RP, RQ, Hk, rk] = TOAR(av, bv, N, n, v0);
    Mn = Vn'*M*Vn;
    En = Vn'*E*Vn;
    Kn = Vn'*K*Vn;
    ln = Vn'*l;
    bn = Vn'*b;

    % reduced transfor function by TOAR
    h_toar= zeros( size(s) );
    for i = 1:length(s)
        h_toar(i) = ln'*( (s(i)^2*Mn+s(i)*En+Kn)\bn );
    end
    hht=[hht;h_toar];

    % MOR by SOAR
    [VSn,PP,TT] = SOAR(av, bv, N, n, v0);
    MSn = VSn'*M*VSn;
    ESn = VSn'*E*VSn;
    KSn = VSn'*K*VSn;
    lSn = VSn'*l;
    bSn = VSn'*b;

    % reduced transfor function by SOAR
    h_soar= zeros( size(s) );
    for i = 1:length(s)
        h_soar(i) = lSn'*( (s(i)^2*MSn+s(i)*ESn+KSn)\bSn );
    end
    hhs=[hhs;h_soar];
end


figure('Position', [0, 0, 1500, 500]); 

% transfer functon plot 
g0=subplot(1,3,1);
set(g0, 'position', [0.06 0.15 .3 .7],'FontSize',16);
semilogy(fq,(abs(h)),'Color',[0,.5,0],'LineStyle','-','LineWidth',2);
hold on;
semilogy(fq,(abs(hhs(3,:))),'Color',[1,0,0],'LineStyle','-.','LineWidth',2);
semilogy(fq,(abs(hht(3,:))),'Color',[0,0,1],'LineStyle',':','LineWidth',2);
legend( 'Exact','SOAR','TOAR','Location','NorthWest');
xlabel('Frequency (Hz)');
ylabel('Magnitudes');
ylim([10^-9,10^-3])
 
% sample a subset of frequency to produce less noisy error plot
inc = 4; % sampling interval
fq = fq([1:inc:end]);
hhs = hhs(:,[1:inc:end]);
hht = hht(:,[1:inc:end]);
h = h([1:inc:end]);

% SOAR plot
g1=subplot(1,3,2);
set(g1, 'position', [0.43 0.15 .25 .7],'FontSize',16);
semilogy(fq,(abs(h-hhs(1,:))./abs(h)),'Color',[0,0,1],'LineStyle',':','LineWidth',2)
hold on;
semilogy(fq,(abs(h-hhs(2,:))./abs(h)),'Color',[0,.5,0],'LineStyle','-.','LineWidth',2)
semilogy(fq,(abs(h-hhs(3,:))./abs(h)),'Color',[1,0,0],'LineStyle','-','LineWidth',2)
legend('k=10','k=20','k=40','Location','NorthWest');
xlabel('Frequency (Hz)');
ylabel('Relative Error');
ylim([10^-14,10^4])
title('SOAR');
set(gca,'YGrid','on','YMinorGrid','on');

% TOAR plot
g2=subplot(1,3,3);
set(g2, 'position', [0.72 0.15 .25 .7],'FontSize',16);
semilogy(fq,(abs(h-hht(1,:))./abs(h)),'Color',[0,0,1],'LineStyle',':','LineWidth',2)
hold on;
semilogy(fq,(abs(h-hht(2,:))./abs(h)),'Color',[0,.5,0],'LineStyle','-.','LineWidth',2)
semilogy(fq,(abs(h-hht(3,:))./abs(h)),'Color',[1,0,0],'LineStyle','-','LineWidth',2)
legend('k=10','k=20','k=40','Location','NorthWest');
xlabel('Frequency (Hz)');
ylim([10^-14,10^4])
title('TOAR');
set(gca,'YGrid','on','YMinorGrid','on');

return
% END
